#!/bin/bash
VERSIONFILE=conf/WCS-OAM.version
INSTALLER_DIR=`pwd`
PRODUCT="FlashphonerWebCallServerOAM"
VERSION=`cat ./$VERSIONFILE`
SUPPORT_URL="http://flashphoner.com"
FLASHPHONER_OAM_HOME="/usr/local/$PRODUCT-$VERSION"
FLASHPHONER_OAM_HOME_WITHOUT_VERSION="/usr/local/$PRODUCT"

MAIN_PID='FlashphonerMainWebCallServer.pid'
OAM_PID='FlashphonerWebCallServer.pid'

function inform_user {
 echo "$1"
}

function check_server_state {
 if [[ -f /var/run/$OAM_PID && $(pgrep -F /var/run/$OAM_PID) ]] ; then
  echo -e "\nYou have to stop $PRODUCT before update\n"
  read -p "Stop now [yes/no] ? " confirm
  if [[ $confirm = y* ]]; then
   $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/bin/wcsoam stop
   echo -e "\n"
  else
   echo -e "\nAbort updating\n "
   exit 1
  fi
 fi
}

function check_write_permissions() {
 if [ ! -w "/usr/local" ]; then	
  echo ""
  echo "ERROR: Write to /usr/local restricted for user running install.sh"
  echo ""
  exit 1;
 fi
}

ALREADY_INSTALLED=false
PRODUCT_VERSION_FILE=$FLASHPHONER_OAM_HOME_WITHOUT_VERSION/$VERSIONFILE

function is_server_installed() {
 if [ -f $PRODUCT_VERSION_FILE ]; then
  PREVIOUS_VERSION=`cat $PRODUCT_VERSION_FILE`
  ALREADY_INSTALLED=true
 fi

 if [ "$PREVIOUS_VERSION" = $VERSION ]; then
  echo "******************************************************"
  echo "*                                                    *"
  echo "*    Flashphoner v.$VERSION already installed        *"
  echo "*                                                    *"
  echo "******************************************************"
  read cont < /dev/tty
  exit 1;
 fi

 inform_user "******************************************************
*                                                    *"

 if [ $ALREADY_INSTALLED != true ]; then
  inform_user "*       Installing $PRODUCT v.$VERSION"
 else
  inform_user "*  Update Flashphoner from $PREVIOUS_VERSION to $VERSION"  
 fi
 inform_user "*                                                    *
*  (C) Flashphoner.com 2010. All rights reserved.    *"
 if [ $ALREADY_INSTALLED != true ]; then
  inform_user "*  To install press ENTER, to abort press CTRL+C.    *"
 else
  check_server_state 
 fi
  inform_user "*                                                    *
*                                        Press ENTER *
******************************************************"

 read cont < /dev/tty
}

function check_system() {
 inform_user "******************************************************
*                                                    *
*              Checking the system                   * 
*                                                    *                                                     
******************************************************
"

 # Checking java
 inform_user "DETECTING java command..."

 JAVA_SYMLINK_BIN=`which java 2>/dev/null`
 if ! test -f "$JAVA_SYMLINK_BIN" ; then
  inform_user "
  ERROR: The Java command (java) could not be found.
  Search path: $PATH
  In most cases this problem can be fixed by adding a symbolic 
  link to the Java command in the /usr/bin directory. 
  To do this first execute the command \"which java\" to identify 
  the full path to the Java executable. Next, create a symbolic
  link to this file with the command
  \"ln -sf [path-to-java] /usr/bin/java\" where [path-to-java] is
  the path returned by the \"which\" command.
  "
  exit 1;
 else
  inform_user "- Java command found successfully."
 fi
 java_ver=`java -version 2>&1| head -1 | cut -d" " -f 3 | tr -d \"`
 java_ver_major=`echo $java_ver | cut -d \. -f 1`
 java_ver_checked=0

 if [[ "$java_ver_major" == "1" ]]; then
  if [[ "$java_ver" > "1.8" ]]; then
   java_ver_checked=1
  fi
 elif [[ "$java_ver_major" > "7" ]] || [[ "$java_ver_major" == "10" ]] || [[ "$java_ver_major" > "10" ]]; then
  java_ver_checked=1
 fi

 if [ $java_ver_checked -eq 0 ]; then
  inform_user "
  ERROR: The Java version should be 8 or higher
  "
  exit 1
 fi

 inform_user ""

# Checking JVM architecture 
 inform_user "DETECTING JVM architecture..."
 java -version 2> tmp.jdk-version 1> /dev/null
 JDK_VERSION=`cat tmp.jdk-version`
 rm tmp.jdk-version
 JVM_ARCH_DIGIT=`echo $JDK_VERSION | grep 64 | sed 's/\(.*\)\(64\)\(.*\)/\2/'`
 if [ "$JVM_ARCH_DIGIT" != "64" ]; then
  JVM_ARCH_DIGIT="32"
 fi
 inform_user "- $JVM_ARCH_DIGIT bit architecture detected successfully.
"

 # Checking hardware platform 
 inform_user "DETECTING hardware platform..."
 HW_PLATFORM=`uname -m`
 HW_PLATFORM_DIGIT=`echo $HW_PLATFORM | sed 's/\(.*\)\(64\)\(.*\)/\2/'`
 if [ "$HW_PLATFORM_DIGIT" != "64" ]; then
  HW_PLATFORM_DIGIT="32"
 fi
 inform_user "- Hardware platform detected successfully: $HW_PLATFORM_DIGIT bit
"

 if [ "$HW_PLATFORM_DIGIT" != "$JVM_ARCH_DIGIT" ]; then
  inform_user "Error. JVM architect: $JVM_ARCH_DIGIT is not compatible with hardware architect: $HW_PLATFORM_DIGIT bit($HW_PLATFORM).
  echo Please, uninstall current JVM/JDK and download and install latest JDK $HW_PLATFORM from oracle.com.
  echo Support - support@flashphoner.com, forum - www.flashphoner.com/forums"
  exit 1;
 fi

 inform_user "****************************************************
*                                                  *
*       The system is ready for installation.      *
*                                                  *
*                                      Press ENTER *
****************************************************
"
}

function copy_files() {
 cd $INSTALLER_DIR

 #Components

 cp -Rf bin $FLASHPHONER_OAM_HOME
 cp -f $INSTALLER_DIR/uninstall.sh $FLASHPHONER_OAM_HOME/bin
 chmod +x $FLASHPHONER_OAM_HOME/bin/uninstall.sh
 chmod +x $FLASHPHONER_OAM_HOME/bin/setenv.sh
 chmod +x $FLASHPHONER_OAM_HOME/bin/wcsoam

 cp -R lib $FLASHPHONER_OAM_HOME


 if [ $ALREADY_INSTALLED != true ]; then
  cp -R conf $FLASHPHONER_OAM_HOME
 fi

 if [ $ALREADY_INSTALLED = true ]; then
  rm -f $FLASHPHONER_OAM_HOME_WITHOUT_VERSION
 fi

 ln -sf $FLASHPHONER_OAM_HOME $FLASHPHONER_OAM_HOME_WITHOUT_VERSION

 echo "$VERSION" > $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/conf/WCS-OAM.version

 #Update log4j2.properties config
 cp -f $INSTALLER_DIR/conf/log4j2.properties $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/conf

 inform_user "- Copying completed."
}

function start_install() {
 if [ $ALREADY_INSTALLED = true ]; then
  inform_user "Starting $PRODUCT update...
  "
 else  
 inform_user "*************************************************************
*                                                           *
*       Starting $PRODUCT installation                      *  
*                                                           *
*************************************************************
"
 fi

 if [ "$JVM_ARCH_DIGIT" = "64" ]; then
  SYSTEM_LIB_DIR=lib64
  JVM_ARCH_X=x8664
 else
  SYSTEM_LIB_DIR=lib
  JVM_ARCH_X=x86         
 fi

 inform_user "COPYING files..."
 
 if [ $ALREADY_INSTALLED = true ]; then
  mv $FLASHPHONER_OAM_HOME_WITHOUT_VERSION-$PREVIOUS_VERSION $FLASHPHONER_OAM_HOME
  #check if Flashphoner is running, if so preserve server.pid
  if [ -f $FLASHPHONER_OAM_HOME/bin/service.pid ]; then
   RUNNING_PID=`cat $FLASHPHONER_OAM_HOME/bin/service.pid`
   rm -rf $FLASHPHONER_OAM_HOME/bin/*
   echo $RUNNING_PID > $FLASHPHONER_OAM_HOME/bin/service.pid
  else
   rm -rf $FLASHPHONER_OAM_HOME/bin
  fi
  rm -rf $FLASHPHONER_OAM_HOME/lib
 else
  if [ ! -d $FLASHPHONER_OAM_HOME ]; then
   mkdir $FLASHPHONER_OAM_HOME
  fi
 fi
 
 copy_files
}

function enable_on_startup {
 
 if [ $ALREADY_INSTALLED != true ]; then
  echo "Do you want to start WebCallServer at server startup? [yes/no]"
  until [ "$res" = "yes" ]; do
   read res
    if [ "$res" = "no" ]; then
     echo "Use $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/bin/wcsoam start to start server"
     res="yes"
    elif [ "$res" = "yes" ]; then

     if [[ `ps --no-headers -o comm 1` == 'systemd' ]]; then
      cd $INSTALLER_DIR
      cp scripts/systemd/wcsoam.service /etc/systemd/system/
      systemctl enable wcsoam
     else 
      cp -f $FLASHPHONER_OAM_HOME/bin/wcsoam /etc/init.d/wcsoam
      if [ -f /proc/version ]; then
       if grep -e "Red Hat\|centos" /proc/version > /dev/null; then
        chkconfig --add wcsoam
        chkconfig --level 345 wcsoam on
       elif grep -e "[Dd]ebian\|[Uu]buntu" /proc/version > /dev/null; then
        update-rc.d -f wcsoam remove
        update-rc.d wcsoam defaults
       fi    
      else 
       echo "Unable to determine your distribution name"
      fi  
     fi

  	else
  	 echo "Please use yes or no"
  	fi
  done
 else 
  if [ -f /etc/init.d/wcsoam ]; then
   cp -f $FLASHPHONER_OAM_HOME/bin/wcsoam /etc/init.d/wcsoam
  fi
 fi
}

function finish_install() {
 inform_user "
************************************************************************************************
*                                                                                              *"
 if [ $ALREADY_INSTALLED = true ]; then
  inform_user "*                                   Update complete!                                           *"
  if [ ! -d "$FLASHPHONER_OAM_HOME_WITHOUT_VERSION/logs" ]; then
   mkdir -p $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/logs
  fi	
  echo "$(date +'%Y-%m-%d %H:%M') Updated from $PREVIOUS_VERSION to $VERSION" >> $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/logs/update.log
 else
  inform_user "*                                   Installation complete!                                     *"
  if [ ! -d "$FLASHPHONER_OAM_HOME_WITHOUT_VERSION/logs" ]; then
     mkdir -p $FLASHPHONER_OAM_HOME_WITHOUT_VERSION/logs
  fi
 fi
 inform_user "*                             Thank you for trying Flashphoner!                                *
*                                                                                              *
*               Please restart $PRODUCT before start the work.                  				 *
*                  Write us with any questions to support@flashphoner.com                      *
*                                                                                              *
*                                                                                              *
*                                                                                              *
************************************************************************************************
"
 read cont < /dev/tty
}

function main() {
 check_write_permissions
 is_server_installed
 check_system
 start_install
 enable_on_startup
 finish_install
}

main

exit 0
